<?php

namespace FiloBlu\PaymentMethodFilter\Observer;

use FiloBlu\PaymentMethodFilter\Helper\PaymentFilter;
use Magento\Framework\DataObject;
use Magento\Quote\Api\CartRepositoryInterface;

class RefiloFilter implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var PaymentFilter
     */
    protected $helper;

    /**
     * @var CartRepositoryInterface
     */
    protected $quoteRepository;

    public function __construct(
        PaymentFilter $helper,
        CartRepositoryInterface $quoteRepository
    ){
        $this->helper = $helper;
        $this->quoteRepository = $quoteRepository;
    }
    public function execute(
        \Magento\Framework\Event\Observer $observer
    ) {
        /**
         * @var DataObject
         */
        $validPaymentMethods = $observer->getData('valid_payment_methods');
        $paymentDetails = $validPaymentMethods->getData('payment_details');
        $cartId = $observer->getData('cart_id');
        $quote = $this->quoteRepository->getActive($cartId);

        $filterdValidPaymentMethods = array();
        $isFilterEnabled = $this->helper->isFilterEnabled();
        $isRemoveCashOnDeliveryPayingByGiftCardEnabled = $this->helper->removeCashOnDeliveryPayingByGiftCard();
        $isFilterByShippingMethodEnabled = $this->helper->isFilterByShippingMethodEnabled();
        $paymentMethodWhitelist = $this->helper->paymentMethodWhitelist();
        $filterConditions = $this->helper->getFilterconditions();

        foreach ($paymentDetails as $method_code)
        {
            if ($isFilterEnabled && !empty($paymentMethodWhitelist)) {
                if (empty($filterConditions)) {
                    if ($this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist)) {
                        $filterdValidPaymentMethods[$method_code] = $method_code;
                    }
                } else {
                    foreach ($filterConditions as $condition) {
                        switch ($condition) {
                            case 'giftcard':
                                if ($this->helper->quoteContainsGiftcard($quote) && $this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist) != FALSE) {
                                    $filterdValidPaymentMethods[$method_code] = $method_code;
                                }
                                break;
                            case 'products': //Products in Cart
                                if ($this->helper->quoteContainsTriggerProducts($quote) && $this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist) != FALSE) {
                                    $filterdValidPaymentMethods[$method_code] = $method_code;
                                }
                                break;
                            case 'product_custom_options': //Product Custom Options in Cart
                                if ($this->helper->quoteContainsTriggerCustomOptions($quote) && $this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist) != FALSE) {
                                    $filterdValidPaymentMethods[$method_code] = $method_code;
                                }
                                break;
                            case 'product_attributes': //Product Attributes in Cart
                                if ($this->helper->quoteContainsTriggerAttributes($quote) && $this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist) != FALSE) {
                                    $filterdValidPaymentMethods[$method_code] = $method_code;
                                }
                                break;
                            case '*': //"any"
                                if ($this->checkMethodAgainstWhitelist($method_code, $paymentMethodWhitelist) != FALSE) {
                                    $filterdValidPaymentMethods[$method_code] = $method_code;
                                }
                            case '':
                            default:
                                break;
                        }
                    }
                }
            }


            if($isRemoveCashOnDeliveryPayingByGiftCardEnabled) {
                if ($this->helper->isPayingWithGiftcard($quote) && $method_code == $this->helper::CASH_ON_DELIVERY_VALUE) {
                    if(in_array($method_code, $filterdValidPaymentMethods)) {
                        unset($filterdValidPaymentMethods[$method_code]);
                    }
                } else {
                    if(!in_array($method_code, $filterdValidPaymentMethods)) {
                        $filterdValidPaymentMethods[$method_code] = $method_code;
                    }
                }
            }

            if ($isFilterByShippingMethodEnabled) {
                $shippingMethod = $quote->getShippingAddress()->getShippingMethod();
                $filterShippingMethods = $this->helper->getFilterByShippingMethodMap();
                foreach($filterShippingMethods as $map_id => $field) {
                    $shipping_code_filter = $field['shipping_code_filter'];
                    if ($shippingMethod == $shipping_code_filter &&
                        $this->checkMethodAgainstWhitelist($method_code, $field['payment_code_filter']) == FALSE) {
                        if(in_array($method_code, $filterdValidPaymentMethods)) {
                            unset($filterdValidPaymentMethods[$method_code]);
                        }
                    } else {
                        $filterdValidPaymentMethods[$method_code] = $method_code;
                    }
                }
            }
        }

        $validPaymentMethods->setData('filtered_payment', $filterdValidPaymentMethods);
    }

    protected function checkMethodAgainstWhitelist($methodCode, $paymentMethodWhitelist) {
        return in_array($methodCode, $paymentMethodWhitelist);
    }
}
